// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// nphkgen
//
// Create OM non periodic housekeeping files.
//
// Giuseppe Vacanti, July 1999
//
#include "common.h"
#include "param.h"
#include "Dal.h"
#include "STime.h"
#include "Xmm.h"
#include "CalOalUtility.h"
#include <errstr.h>
#include <SasDalUtils.h>
#include "nphkgen.h"

void nphkgen()
{

  PROC
  errstr << "nphkgen running" << Sas::message(Msg::App, Msg::Verbose);

  STime start(stringParameter("tstart"));
  STime stop(stringParameter("tstop"));
  
  unsigned int nrows = 2;

  DataSet * set = dataSetServer->open(stringParameter("nphkf"), Dal::Create);

  * set->addAttribute("ORIGIN", "processing site") = stringParameter("origin");
  * set->addAttribute("CATEGORY") = stringParameter("category");
  * set->addAttribute("FILENAME") = stringParameter("nphkf");

  Table * tab = set->addTable(stringParameter("extname"), nrows);

  * tab->addAttribute("TELESCOP") = "XMM";
  * tab->addAttribute("INSTRUME") = stringParameter("instrument");
  * tab->addAttribute("DATATYPE") = "NPERHK.EL";
  * tab->addAttribute("OBS_ID", "Observation identifier") = stringParameter("obsid");
  * tab->addAttribute("EXP_ID", "Exposure identifier") = stringParameter("expid");
  * tab->addAttribute("DATE-OBS") = stringParameter("tstart");
  * tab->addAttribute("DATE-END") = stringParameter("tstop");
  
  vector<unsigned long> size; size.push_back(19);

  Column * time = tab->addColumn("TIME",
				 Dal::DString,
				 "time tag",
				 "ERT",
				 size);

  size[0] = 1;
  Column * ftcoarse = tab->addColumn("FTCOARSE",
				     Dal::Int32,
				     "telemetry packet header coarse time",
				     "s",
				     size);
  
  Column * ftfine = tab->addColumn("FTFINE",
				   Dal::Uint16,
				   "telemetry packet header fine time",
				   "",
				   size);
  
  Column * sid = tab->addColumn("SID",
				Dal::Int16,
				"structure identifier",
				"",
				size);
  
  Column * type = tab->addColumn("TYPE",
				 Dal::Int32,
				 "report identifier",
				 "",
				 size);

  Column * par1 = tab->addColumn("PAR1",
				 Dal::Int32,
				 "parameter 1",
				 "",
				 size);

  Column * par2 = tab->addColumn("PAR2",
				 Dal::Int32,
				 "parameter 2",
				 "",
				 size);

  Column * par3 = tab->addColumn("PAR3",
				 Dal::Int32,
				 "parameter 3",
				 "",
				 size);

  Column * par4 = tab->addColumn("PAR4",
				 Dal::Int32,
				 "parameter 4",
				 "",
				 size);

  Column * par5 = tab->addColumn("PAR5",
				 Dal::Int32,
				 "parameter 5",
				 "",
				 size);

  
  ColumnData * ptr;

  ptr = ftcoarse->data();
  int32 * pftcoarse = ptr->int32Data();
  STime tstop(stringParameter("tstop"));
  STime tstart(stringParameter("tstart"));
  pftcoarse[0] = 0;
  pftcoarse[1] = static_cast<int>(tstop - tstart);

  ptr = ftfine->data();
  uint16 * pftfine = ptr->uint16Data();
  pftfine[0] = 0;
  pftfine[1] = 0;

  ptr = type->data();
  int32 * ptype = ptr->int32Data();
  ptype[0] = 0xa513;
  ptype[1] = 0xa514;

  ptr = par2->data();
  int32 * ppar2 = ptr->int32Data();
  ppar2[0] = pftfine[0] + static_cast<int>(pow(2.,10)*pftcoarse[0]);
  ppar2[1] = pftfine[1] + static_cast<int>(pow(2.,10)*pftcoarse[1]);

  using namespace SasDalUtils;
  ptr = time->data();
  setStringColumnRow(ptr, 0, stringParameter("tstart"));
  setStringColumnRow(ptr, 1, stringParameter("tstop"));  

  ptr = sid->data();
  int16 * psid = ptr->int16Data();
  psid[0] = 148;
  psid[1] = 149;

  ptr = par1->data();
  ptr = par3->data();
  ptr = par4->data();
  ptr = par5->data();

  delete ptr;

  dataSetServer->close(set);

  ENDPROC
}


