! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: rgsauxtable_mod
!
! Authors:  Peter Videler - SOC/ESTEC
!          
! Description: Fix RGS Auxlillary table for int16 overflow
! 
! Comments: 
! 
! $Id: rgsauxtable_mod.f90,v 1.2 2000/07/03 09:31:03 gvacanti Exp $
!- 
module rgsauxtable_mod

  use dal
  use param  
  use saslib
  
  implicit none 
  
  private 

  public rgsauxtable

contains 
  
  !---------------------------------------------------------------------
  subroutine rgsauxtable
  !---------------------------------------------------------------------

    use sasdal
    type(DataSetT)  :: set  
    type(TableT)    :: tbl  
    type(ColumnT)   :: col
    integer(kind=int32), dimension(:), pointer :: frames
    integer :: row

    set = dataSet(stringParameter("set"), modify)
    if (hasBlock(set, 'R1AUX1')) then
      tbl = table(set,'R1AUX1' , modify)     ! RGS1
    else
      tbl = table(set,'R2AUX1' , modify)     ! RGS2
    endif
        
    col = column(tbl, 'FRAME', modify)
    frames => int32Data(col)

    if (.not. monotonicIncreasing(frames)) then
       call message(appMsg, verbose,"Frame numbers in AUX table not increasing. &
            & Sorting table "//trim(name(set))) 
       call tableSort(tbl, col, sortorder=1)   ! from saslib/lib/sasdal
       frames => int32Data(col)
    endif

    ! Stupit algorithm. Deleting rows one at a time.
    do 
       row = firstDuplicate(frames)
       if (row == 0) then
          EXIT
       else
          write(errstr,*) "Duplicate frame number found. Deleting row ", &
               row, "from table "//trim(name(tbl))
          call message(appMsg, verbose) 
          call deleteRows(table=tbl, from=row-1, count=1)    ! dal row number starts at 1
          frames => int32Data(col)
       endif
    enddo
    
    call release (set)

  end subroutine rgsauxtable

  !------------------------------------------------------------
  function firstDuplicate(array) result(line)
    !------------------------------------------------------------

   ! returns first array element for which array(i)=array(i-1)
   ! otherwise return 0

    integer, dimension(:), intent(in)  :: array
    integer                            :: line, i

    line=0
    do i= 2, size(array)
       if (array(i) == array(i-1)) then
          line=i
          EXIT
       endif
    enddo
  end function firstDuplicate
  
end module rgsauxtable_mod
