! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
module wdgen_mod
  use dal
  use param
  use errorHandling
  implicit none
contains
subroutine wdgen

  character(len=1024) :: filename
  integer :: x0, y0, dx, dy, binax1, binax2
  type(dataSetT) :: windowSet
  type(TableT) :: windowTable
  type(columnT) :: windowColumn
  integer(kind=int32), pointer :: iCol(:),iVector(:,:)
  integer :: dims(1), i

  !! Added by GV -- 1998 NOvember 24
  ! read parameters
  filename = stringParameter("wdf")
  x0 = intParameter("xwin0")
  y0 = intParameter("ywin0")
  dx = intParameter("dwinx")
  dy = intParameter("dwiny")
  binax1 = intParameter("binax1")
  binax2 = intParameter("binax2")
  !!


  dims(1) = 16

  i = len_trim(filename)

  windowSet = dataSet(filename, create)
  call setAttribute(windowSet,"FILENAME",filename(i-30:i))
  

  windowTable = addTable(windowSet, "OMWDX1", 1)

  windowColumn = addColumn(windowTable, "DETWIND", integer32)
  iCol => int32data(windowColumn)
  icol(0) = 1
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SCIWIN", integer32)
  iCol => int32data(windowColumn)
  icol(0) = 1
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWX0", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)


  windowColumn = addColumn(windowTable, "DWY0", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWXSIZE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWYSIZE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWMMWID", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWKOMPX", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWBB", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWAA", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "DWKOMPY", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWX0", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector(0,0) = x0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWY0", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector(0,0) = y0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWXSIZE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector(0,0) = dx
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWYSIZE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector(0,0) = dy
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWMODE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0 ! this is valid for image mode. fast mode would be 1. guide stars would be 15.
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWP1", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector(0,0) = binax1
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWP2", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector(0,0) = binax2
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWP3", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWP4", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWBASE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "SWMMWID", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "MWX0", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "MWY0", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "MWXSIZE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "MWYSIZE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  windowColumn = addColumn(windowTable, "MWBASE", integer32, dimensions=dims)
  iVector => int32VectorData(windowColumn)
  iVector  = 0
  call release(windowColumn)

  call release(windowTable)
  call release(windowSet)

end subroutine wdgen
end module wdgen_mod
