// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine) 
// September 2000
//
// Aitor Ibarra 22/10/2004
// Based on the IppvTranslator, we have implemented a translator
// between the Scisim output and the SAS infrastructure. 

#include "ScisimToSasTranslator.h"
#include <errstr.h>
#include <iostream>
#include <vector>


template<class T> ScisimToSasTranslator<T>::ScisimToSasTranslator()
  : _map(map<T,string>())
{

}

template<class T> ScisimToSasTranslator<T>::~ScisimToSasTranslator()
{

}

template<class T> const string& ScisimToSasTranslator<T>::look_up(const T& t) const
{
  if(_map.find(t) == _map.end())
    //    throw "No look up possible [" + _name + "]";
    errstr << "Filter name introduced not valid " << Sas::error("InvalidFilterName");	  
  return _map[t];
}

template<class T> void ScisimToSasTranslator<T>::print(ostream& os) const
{

  os << "Translator's name: " << _name << endl;
  for(typename map<T,string>::const_iterator k = _map.begin(); k != _map.end(); k++)
    os << (*k).first << " = " << (*k).second << endl;

}

template<class T> bool ScisimToSasTranslator<T>::has_look_up() const
{
  
  return _map.size() == 0 ? false : true;
}

//

//template class ScisimToSasTranslator<int8>;
//template class ScisimToSasTranslator<int16>;
//template class ScisimToSasTranslator<uint16>;
//template class ScisimToSasTranslator<int32>;
//template class ScisimToSasTranslator<uint32>;
template class ScisimToSasTranslator<double>;
//template class ScisimToSasTranslator<real64>;
template class ScisimToSasTranslator<string>;


template<> FilterWheelTranslator<string>::FilterWheelTranslator() {
  _name = "Filter Mode";
  // Fix typos in HK
  _map["open"] = "Open";
  _map["thick"] = "Thick";
  _map["medium"] = "Medium";
  // FIXME: how many typos are there?? Is it THIN or THIN2??
  //_map["FILTRB-THIN"] = "Thin2";
  //_map["FILTRB-THIN2"] = "Thin2";
  _map["thin"] = "Thin1";
  _map["closed"] = "Closed";
  _map["none"] = "Filter_None";
  _map["UNKNOWN"] = "UNKNOWN";
  //  _map["NO STOP POS."] = "Filter_None";
  
  errstr << "Setting up " + _name + " with the following:" << endl;
  for(map<string,string>::const_iterator k=_map.begin();
      k!=_map.end(); ++k)
    errstr << (*k).first << " -> " << (*k).second << endl;
  errstr <<Sas::message(Msg::AppLib,Msg::Noisy);
  
}

