// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// 1998-1999, Giuseppe Vacanti (XMM-SOC ESA)
//
// odffix scans a given directory for ODF components. These are then
// listed in the ODF summary file.
//
// The bulk of the work is done in OdfObject.
//

#include "param.h"

#include "errstr.h"
#include "Tracer.h"

#include "OdfObject.h"

#include "odffix.h"
#include "ahfgen.h"
#include "tcfgen.h"
#include "tcfgen.h"

void odffix () {
  PROC
    
  OdfObject * o;
  if(booleanParameter("findnominalpointing")) // from ATS file
    o = new OdfObject(stringParameter("odfdir"));
  else
    o = new OdfObject(realParameter("ra"), realParameter("dec"),
		      realParameter("posangle"), stringParameter("odfdir"));
  
  if(o->numberOfFiles() == 0){
    delete o;
    errstr << "No ODF component found" << Sas::error("NoOdfComponent");
  }

  o->generateAuxiliaryFiles();
  o->generateHousekeepingFiles();
  o->generateAttitudeHistoryFile();
  o->generateTimeCorrelationFile();
  o->generateSummaryFile();

  delete o;

  ENDPROC
}





